using InternetShopContracts.DataSearchModels;
using InternetShopContracts.DataViewModels;
using InternetShopContracts.LogicsContracts;
using InternetShopForms.Orders;
using InternetShopForms.Products;

namespace InternetShopForms
{
    public partial class FormOrdersList : Form
    {
        private IOrderLogic _orderLogic;
        public FormOrdersList(IOrderLogic orderLogic)
        {
            InitializeComponent();
            _orderLogic = orderLogic;

            List<(string, string, float)> configureColumns = [
                ("ID", "Id", 1.0f),
                (" ", "CustomerFIO", 1.0f),
                ("", "ProductsString", 1.0f),
                ("Email ", "CustomerEmail", 1.0f),
            ];

            tableComponent.ConfigureColumns(configureColumns);
            tableComponent.dataGridView1.AllowUserToDeleteRows = false;

            tableComponent.dataGridView1.Columns[0].Visible = false;
            KeyPreview = true;
        }

        private void productsToolStripMenuItem_Click(object sender, EventArgs e)
        {
            var service = Program.ServiceProvider?.GetService(typeof(FormProductsList));
            if (service is FormProductsList form)
            {
                form.ShowDialog();
            }
        }

        private void FormOrdersList_Load(object sender, EventArgs e)
        {
            LoadData();
        }

        private void LoadData()
        {
            try
            {
                var orders = _orderLogic.ReadList();
                tableComponent.FillData(orders);
            }
            catch (Exception ex)
            {
                MessageBox.Show("    :\n" + ex.Message, "", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }

        private void FormOrdersList_KeyDown(object sender, KeyEventArgs e)
        {
            if (e.Control)
            {
                switch (e.KeyCode)
                {
                    case Keys.A:
                        CreateOrder();
                        e.SuppressKeyPress = true;
                        break;
                    case Keys.U:
                        EditOrder();
                        e.SuppressKeyPress = true;
                        break;
                    case Keys.D:
                        DeleteOrder();
                        e.SuppressKeyPress = true;
                        break;
                    case Keys.S:
                        CreateWord();
                        e.SuppressKeyPress = true;
                        break;
                    case Keys.C:
                        CreateExcel();
                        e.SuppressKeyPress = true;
                        break;
                    case Keys.T:
                        CreatePDF();
                        e.SuppressKeyPress = true;
                        break;
                }
            }
        }

        private void CreateOrder()
        {
            var service = Program.ServiceProvider?.GetService(typeof(FormOrderEdit));
            if (!(service is FormOrderEdit form))
            {
                return;
            }

            if (form.ShowDialog() == DialogResult.OK)
            {
                LoadData();
            }
        }

        private void EditOrder()
        {
            OrderViewModel? selectedOrder = null;
            try
            {
                selectedOrder = tableComponent.GetSelectedObject<OrderViewModel>();
            }
            catch (Exception ex)
            {
                MessageBox.Show(" ");
                return;
            }
            var service = Program.ServiceProvider?.GetService(typeof(FormOrderEdit));
            if (!(service is FormOrderEdit form))
            {
                return;
            }
            form.OrderId = Convert.ToInt32(selectedOrder.Id);
            if (form.ShowDialog() == DialogResult.OK)
            {
                LoadData();
            }
        }

        private void DeleteOrder()
        {
            try
            {
                OrderViewModel? selectedOrder = null;
                try
                {
                    selectedOrder = tableComponent.GetSelectedObject<OrderViewModel>();
                }
                catch (Exception ex)
                {
                    MessageBox.Show(" ");
                    return;
                }

                var result = MessageBox.Show(
                    "   ?",
                    "",
                    MessageBoxButtons.YesNo,
                    MessageBoxIcon.Question
                );

                if (result == DialogResult.Yes)
                {
                    if (_orderLogic.Delete(new OrderSearchModel { Id = selectedOrder.Id }))
                    {
                        MessageBox.Show(" ");
                    }
                    else
                    {
                        throw new Exception(" ");
                    }
                    LoadData();
                }
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message);
            }
        }

        private enum ReportType
        {
            Word = 0,
            Excel = 1,
            PDF = 2
        }

        private string GetFileFilter(ReportType reportType)
        {
            if (reportType == ReportType.Excel)
            {
                return "Excel files (*.xlsx)|*.xlsx";
            }
            else if (reportType == ReportType.PDF)
            {
                return "PDF files (*.pdf)|*.pdf";
            }
            else
            {
                return "Word files (*.docx)|*.docx";
            }
        }

        private string? GetExportFileName(ReportType reportType)
        {
            using (SaveFileDialog openFileDialog = new SaveFileDialog())
            {
                openFileDialog.InitialDirectory = "d:\\tmp";
                openFileDialog.Filter = GetFileFilter(reportType) + "|All files (*.*)|*.*";
                openFileDialog.FilterIndex = 1;
                openFileDialog.RestoreDirectory = true;

                if (openFileDialog.ShowDialog() == DialogResult.OK)
                {
                    return openFileDialog.FileName;
                }
            }
            return null;
        }

        private void CreateWord()
        {
            string? exportFileName = GetExportFileName(ReportType.Word);
            if (string.IsNullOrEmpty(exportFileName))
            {
                return;
            }
            var orders = _orderLogic.ReadList();
            try
            {
                bigTextWordNoVisibleComponent.CreateAndSaveDocument(exportFileName, "  ", );
                MessageBox.Show("  ", " ", MessageBoxButtons.OK, MessageBoxIcon.Information);
            }
            catch (Exception ex)
            {
                MessageBox.Show("    :\n" + ex.Message, " ", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }

        private void CreateExcel()
        {
            string? exportFileName = GetExportFileName(ReportType.Excel);
            if (string.IsNullOrEmpty(exportFileName))
            {
                return;
            }
            var orders = _orderLogic.ReadList();
            try
            {
                componentExcelWithImage.CreateExcelWithImages(exportFileName, "", orders.Select(x => x.ImagePath).ToArray());
                MessageBox.Show("  ", " ", MessageBoxButtons.OK, MessageBoxIcon.Information);
            }
            catch (Exception ex)
            {
                MessageBox.Show("    :\n" + ex.Message, " ", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
            
        }

        private void CreatePDF()
        {
            string? exportFileName = GetExportFileName(ReportType.PDF);
            if (string.IsNullOrEmpty(exportFileName))
            {
                return;
            }
            // todo
        }

        private void reportOrderImagesExcelToolStripMenuItem_Click(object sender, EventArgs e)
        {
            CreateExcel();
        }

        private void reportOrdersWordToolStripMenuItem_Click(object sender, EventArgs e)
        {
            CreateWord();
        }

        private void reportProductsPDFToolStripMenuItem_Click(object sender, EventArgs e)
        {
            CreatePDF();
        }
    }
}
